@@ -2,7 +2,7 @@ from __future__ import absolute_import, division, print_function
 
 from _echopype_version import version as __version__  # noqa
 
-from . import calibrate, clean, commongrid, consolidate, mask, preprocess, utils
+from . import calibrate, clean, commongrid, consolidate, mask, utils
 from .convert.api import open_raw
 from .echodata.api import open_converted
 from .echodata.combine import combine_echodata
@@ -21,9 +21,9 @@ __all__ = [
     "commongrid",
     "consolidate",
     "mask",
+    "metrics",
     "open_converted",
     "open_raw",
-    "preprocess",
     "utils",
     "verbose",
 ]
