@@ -1,28 +1,50 @@
 import abc
 
 from ..echodata import EchoData
+from ..utils.log import _init_logger
+from .ecs import ECSParser
+
+logger = _init_logger(__name__)
 
 
 class CalibrateBase(abc.ABC):
     """Class to handle calibration for all sonar models."""
 
-    def __init__(self, echodata: EchoData, env_params=None, cal_params=None):
+    def __init__(self, echodata: EchoData, env_params=None, cal_params=None, ecs_file=None):
         self.echodata = echodata
         self.sonar_type = None
+        self.ecs_file = ecs_file
+        self.ecs_dict = {}
 
-        if env_params is None:
+        # Set ECS to overwrite user-provided dict
+        if self.ecs_file is not None:
+            if env_params is not None or cal_params is not None:
+                logger.warning(
+                    "The ECS file takes precedence when it is provided. "
+                    "Parameter values provided in 'env_params' and 'cal_params' will not be used!"
+                )
+
+            # Parse ECS file to a dict
+            ecs = ECSParser(self.ecs_file)
+            ecs.parse()
+            self.ecs_dict = ecs.get_cal_params()  # apply ECS hierarchy
             self.env_params = {}
-        elif isinstance(env_params, dict):
-            self.env_params = env_params
-        else:
-            raise ValueError("'env_params' has to be None or a dict")
-
-        if cal_params is None:
             self.cal_params = {}
-        elif isinstance(cal_params, dict):
-            self.cal_params = cal_params
+
         else:
-            raise ValueError("'cal_params' has to be None or a dict")
+            if env_params is None:
+                self.env_params = {}
+            elif isinstance(env_params, dict):
+                self.env_params = env_params
+            else:
+                raise ValueError("'env_params' has to be None or a dict")
+
+            if cal_params is None:
+                self.cal_params = {}
+            elif isinstance(cal_params, dict):
+                self.cal_params = cal_params
+            else:
+                raise ValueError("'cal_params' has to be None or a dict")
 
         # range_meter is computed in compute_Sv/TS in child class
         self.range_meter = None
