@@ -203,12 +203,14 @@ class SetGroupsAd2cp(SetGroupsBase):
             Union[Tuple[List[str], np.ndarray, Dict[str, str]], Tuple[Tuple[()], None]],
         ] = {
             var_name: (
-                [dim.dimension_name() for dim in dims[field_name]],
-                combined_fields[field_name],
-                attrs.get(field_name, {}),
+                (
+                    [dim.dimension_name() for dim in dims[field_name]],
+                    combined_fields[field_name],
+                    attrs.get(field_name, {}),
+                )
+                if field_exists[field_name]
+                else ((), None)
             )
-            if field_exists[field_name]
-            else ((), None)
             for field_name, var_name in var_names.items()
         }  # type: ignore
         coords: Dict[str, np.ndarray] = dict()
@@ -245,13 +247,6 @@ class SetGroupsAd2cp(SetGroupsBase):
                 "roll": "roll",
             }
         )
-        ds = ds.assign_attrs(
-            {
-                "platform_name": self.ui_param["platform_name"],  # type: ignore
-                "platform_type": self.ui_param["platform_type"],  # type: ignore
-                "platform_code_ICES": self.ui_param["platform_code_ICES"],  # type: ignore
-            }
-        )
         return set_time_encodings(ds)
 
     def set_beam(self) -> List[xr.Dataset]:
