@@ -1,13 +1,13 @@
 import abc
-from typing import List, Set, Tuple
+import warnings
+from typing import List, Set
 
-import dask.array
 import numpy as np
 import pynmea2
 import xarray as xr
 
 from ..echodata.convention import sonarnetcdf_1
-from ..utils.coding import COMPRESSION_SETTINGS, set_time_encodings
+from ..utils.coding import COMPRESSION_SETTINGS, DEFAULT_TIME_ENCODING, set_time_encodings
 from ..utils.prov import echopype_prov_attrs, source_files_vars
 
 NMEA_SENTENCE_DEFAULT = ["GGA", "GLL", "RMC"]
@@ -27,7 +27,6 @@ class SetGroupsBase(abc.ABC):
         compress=True,
         overwrite=True,
         params=None,
-        parsed2zarr_obj=None,
     ):
         # parser object ParseEK60/ParseAZFP/etc...
         self.parser_obj = parser_obj
@@ -41,10 +40,6 @@ class SetGroupsBase(abc.ABC):
         self.engine = engine
         self.compress = compress
         self.overwrite = overwrite
-        self.ui_param = params
-
-        # parsed data written directly to zarr object
-        self.parsed2zarr_obj = parsed2zarr_obj
 
         if not self.compress:
             self.compression_settings = None
@@ -71,7 +66,6 @@ class SetGroupsBase(abc.ABC):
             "summary": "",
             "title": "",
             "date_created": np.datetime_as_string(date_created, "s") + "Z",
-            "survey_name": self.ui_param["survey_name"],
         }
         # Save
         ds = xr.Dataset()
@@ -113,20 +107,45 @@ class SetGroupsBase(abc.ABC):
         """Set the Platform group."""
         raise NotImplementedError
 
+    def _nan_timestamp_handler(self, time_val) -> List:
+        """
+        Replace nan in time coordinate to avoid xarray warning.
+        """
+        if len(time_val) == 1 and np.isnan(time_val[0]):
+            # set time_val to earliest ping_time among all channels
+            if self.sonar_model in ["EK60", "ES70", "EK80", "ES80", "EA640"]:
+                return [np.array([v[0] for v in self.parser_obj.ping_time.values()]).min()]
+            elif self.sonar_model == "AZFP":
+                return [self.parser_obj.ping_time[0]]
+            else:
+                return NotImplementedError(
+                    f"Nan timestamp handling has not been implemented for {self.sonar_model}!"
+                )
+        else:
+            return time_val
+
     def set_nmea(self) -> xr.Dataset:
         """Set the Platform/NMEA group."""
         # Save nan if nmea data is not encoded in the raw file
         if len(self.parser_obj.nmea["nmea_string"]) != 0:
-            # Convert np.datetime64 numbers to seconds since 1900-01-01 00:00:00Z
+            # Convert np.datetime64 numbers to nanoseconds since 1970-01-01 00:00:00Z
             # due to xarray.to_netcdf() error on encoding np.datetime64 objects directly
-            time = (
-                self.parser_obj.nmea["timestamp"] - np.datetime64("1900-01-01T00:00:00")
-            ) / np.timedelta64(1, "s")
+            # print(np.array(self.parser_obj.nmea["timestamp"])[idx_loc].shape)
+            time, _, _ = xr.coding.times.encode_cf_datetime(
+                self.parser_obj.nmea["timestamp"],
+                **{
+                    "units": DEFAULT_TIME_ENCODING["units"],
+                    "calendar": DEFAULT_TIME_ENCODING["calendar"],
+                },
+            )
             raw_nmea = self.parser_obj.nmea["nmea_string"]
         else:
             time = [np.nan]
             raw_nmea = [np.nan]
 
+        # Handle potential nan timestamp for time
+        time = self._nan_timestamp_handler(time)
+
         ds = xr.Dataset(
             {
                 "NMEA_datagram": (
@@ -175,30 +194,42 @@ class SetGroupsBase(abc.ABC):
                 pynmea2.ParseError,
             ):
                 nmea_msg.append(None)
-        lat = (
-            np.array([x.latitude if hasattr(x, "latitude") else np.nan for x in nmea_msg])
-            if nmea_msg
-            else [np.nan]
-        )
-        lon = (
-            np.array([x.longitude if hasattr(x, "longitude") else np.nan for x in nmea_msg])
-            if nmea_msg
-            else [np.nan]
-        )
+        if nmea_msg:
+            lat, lon = [], []
+            for x in nmea_msg:
+                try:
+                    lat.append(x.latitude if hasattr(x, "latitude") else np.nan)
+                except ValueError as ve:
+                    lat.append(np.nan)
+                    warnings.warn(
+                        "At least one latitude entry is problematic and "
+                        f"are assigned None in the converted data: {str(ve)}"
+                    )
+                try:
+                    lon.append(x.longitude if hasattr(x, "longitude") else np.nan)
+                except ValueError as ve:
+                    lon.append(np.nan)
+                    warnings.warn(
+                        f"At least one longitude entry is problematic and "
+                        f"are assigned None in the converted data: {str(ve)}"
+                    )
+        else:
+            lat, lon = [np.nan], [np.nan]
         msg_type = (
-            np.array([x.sentence_type if hasattr(x, "sentence_type") else np.nan for x in nmea_msg])
+            [x.sentence_type if hasattr(x, "sentence_type") else np.nan for x in nmea_msg]
             if nmea_msg
             else [np.nan]
         )
-        time1 = (
-            (
-                np.array(self.parser_obj.nmea["timestamp"])[idx_loc]
-                - np.datetime64("1900-01-01T00:00:00")
+        if nmea_msg:
+            time1, _, _ = xr.coding.times.encode_cf_datetime(
+                np.array(self.parser_obj.nmea["timestamp"])[idx_loc],
+                **{
+                    "units": DEFAULT_TIME_ENCODING["units"],
+                    "calendar": DEFAULT_TIME_ENCODING["calendar"],
+                },
             )
-            / np.timedelta64(1, "s")
-            if nmea_msg
-            else [np.nan]
-        )
+        else:
+            time1 = [np.nan]
 
         return time1, msg_type, lat, lon
 
@@ -331,220 +362,3 @@ class SetGroupsBase(abc.ABC):
 
         self._add_ping_time_dim(ds, beam_ping_time_names, ping_time_only_names)
         self._add_beam_dim(ds, beam_only_names, beam_ping_time_names)
-
-    def _get_channel_ids(self, chan_str: np.ndarray) -> List[str]:
-        """
-        Obtains the channel IDs associated with ``chan_str``.
-
-        Parameters
-        ----------
-        chan_str : np.ndarray
-            A numpy array of strings corresponding to the
-            keys of ``config_datagram["transceivers"]``
-
-        Returns
-        -------
-        A list of strings representing the channel IDS
-        """
-        if self.sonar_model in ["EK60", "ES70"]:
-            return [
-                self.parser_obj.config_datagram["transceivers"][int(i)]["channel_id"]
-                for i in chan_str
-            ]
-        else:
-            return [
-                self.parser_obj.config_datagram["configuration"][i]["channel_id"] for i in chan_str
-            ]
-
-    def _get_power_dataarray(self, zarr_path: str) -> xr.DataArray:
-        """
-        Constructs a DataArray from a Dask array for the power
-        data.
-
-        Parameters
-        ----------
-        zarr_path: str
-            Path to the zarr file that contain the power data
-
-        Returns
-        -------
-        DataArray named "backscatter_r" representing the
-        power data.
-        """
-
-        # collect variables associated with the power data
-        power = dask.array.from_zarr(zarr_path, component="power/power")
-
-        pow_time_path = "power/" + self.parsed2zarr_obj.power_dims[0]
-        pow_chan_path = "power/" + self.parsed2zarr_obj.power_dims[1]
-        power_time = dask.array.from_zarr(zarr_path, component=pow_time_path).compute()
-        power_channel = dask.array.from_zarr(zarr_path, component=pow_chan_path).compute()
-
-        # obtain channel names for power data
-        pow_chan_names = self._get_channel_ids(power_channel)
-
-        backscatter_r = xr.DataArray(
-            data=power,
-            coords={
-                "ping_time": (
-                    ["ping_time"],
-                    power_time,
-                    self._varattrs["beam_coord_default"]["ping_time"],
-                ),
-                "channel": (
-                    ["channel"],
-                    pow_chan_names,
-                    self._varattrs["beam_coord_default"]["channel"],
-                ),
-                "range_sample": (
-                    ["range_sample"],
-                    np.arange(power.shape[2]),
-                    self._varattrs["beam_coord_default"]["range_sample"],
-                ),
-            },
-            name="backscatter_r",
-            attrs={"long_name": "Backscatter power", "units": "dB"},
-        )
-
-        return backscatter_r
-
-    def _get_angle_dataarrays(self, zarr_path: str) -> Tuple[xr.DataArray, xr.DataArray]:
-        """
-        Constructs the DataArrays from Dask arrays associated
-        with the angle data.
-
-        Parameters
-        ----------
-        zarr_path: str
-            Path to the zarr file that contains the angle data
-
-        Returns
-        -------
-        DataArrays named "angle_athwartship" and "angle_alongship",
-        respectively, representing the angle data.
-        """
-
-        # collect variables associated with the angle data
-        angle_along = dask.array.from_zarr(zarr_path, component="angle/angle_alongship")
-        angle_athwart = dask.array.from_zarr(zarr_path, component="angle/angle_athwartship")
-
-        ang_time_path = "angle/" + self.parsed2zarr_obj.angle_dims[0]
-        ang_chan_path = "angle/" + self.parsed2zarr_obj.angle_dims[1]
-        angle_time = dask.array.from_zarr(zarr_path, component=ang_time_path).compute()
-        angle_channel = dask.array.from_zarr(zarr_path, component=ang_chan_path).compute()
-
-        # obtain channel names for angle data
-        ang_chan_names = self._get_channel_ids(angle_channel)
-
-        array_coords = {
-            "ping_time": (
-                ["ping_time"],
-                angle_time,
-                self._varattrs["beam_coord_default"]["ping_time"],
-            ),
-            "channel": (
-                ["channel"],
-                ang_chan_names,
-                self._varattrs["beam_coord_default"]["channel"],
-            ),
-            "range_sample": (
-                ["range_sample"],
-                np.arange(angle_athwart.shape[2]),
-                self._varattrs["beam_coord_default"]["range_sample"],
-            ),
-        }
-
-        angle_athwartship = xr.DataArray(
-            data=angle_athwart,
-            coords=array_coords,
-            name="angle_athwartship",
-            attrs={
-                "long_name": "electrical athwartship angle",
-                "comment": (
-                    "Introduced in echopype for Simrad echosounders. "  # noqa
-                    + "The athwartship angle corresponds to the major angle in SONAR-netCDF4 vers 2. "  # noqa
-                ),
-            },
-        )
-
-        angle_alongship = xr.DataArray(
-            data=angle_along,
-            coords=array_coords,
-            name="angle_alongship",
-            attrs={
-                "long_name": "electrical alongship angle",
-                "comment": (
-                    "Introduced in echopype for Simrad echosounders. "  # noqa
-                    + "The alongship angle corresponds to the minor angle in SONAR-netCDF4 vers 2. "  # noqa
-                ),
-            },
-        )
-
-        return angle_athwartship, angle_alongship
-
-    def _get_complex_dataarrays(self, zarr_path: str) -> Tuple[xr.DataArray, xr.DataArray]:
-        """
-        Constructs the DataArrays from Dask arrays associated
-        with the complex data.
-
-        Parameters
-        ----------
-        zarr_path: str
-            Path to the zarr file that contains the complex data
-
-        Returns
-        -------
-        DataArrays named "backscatter_r" and "backscatter_i",
-        respectively, representing the complex data.
-        """
-
-        # collect variables associated with the complex data
-        complex_r = dask.array.from_zarr(zarr_path, component="complex/backscatter_r")
-        complex_i = dask.array.from_zarr(zarr_path, component="complex/backscatter_i")
-
-        comp_time_path = "complex/" + self.parsed2zarr_obj.complex_dims[0]
-        comp_chan_path = "complex/" + self.parsed2zarr_obj.complex_dims[1]
-        complex_time = dask.array.from_zarr(zarr_path, component=comp_time_path).compute()
-        complex_channel = dask.array.from_zarr(zarr_path, component=comp_chan_path).compute()
-
-        # obtain channel names for complex data
-        comp_chan_names = self._get_channel_ids(complex_channel)
-
-        array_coords = {
-            "ping_time": (
-                ["ping_time"],
-                complex_time,
-                self._varattrs["beam_coord_default"]["ping_time"],
-            ),
-            "channel": (
-                ["channel"],
-                comp_chan_names,
-                self._varattrs["beam_coord_default"]["channel"],
-            ),
-            "range_sample": (
-                ["range_sample"],
-                np.arange(complex_r.shape[2]),
-                self._varattrs["beam_coord_default"]["range_sample"],
-            ),
-            "beam": (
-                ["beam"],
-                np.arange(start=1, stop=complex_r.shape[3] + 1).astype(str),
-                self._varattrs["beam_coord_default"]["beam"],
-            ),
-        }
-
-        backscatter_r = xr.DataArray(
-            data=complex_r,
-            coords=array_coords,
-            name="backscatter_r",
-            attrs={"long_name": "Real part of backscatter power", "units": "V"},
-        )
-
-        backscatter_i = xr.DataArray(
-            data=complex_i,
-            coords=array_coords,
-            name="backscatter_i",
-            attrs={"long_name": "Imaginary part of backscatter power", "units": "V"},
-        )
-
-        return backscatter_r, backscatter_i
