@@ -1,10 +1,10 @@
 import datetime
-import shutil
 import warnings
 from html import escape
 from pathlib import Path
-from typing import TYPE_CHECKING, Any, Dict, Optional, Set, Tuple
+from typing import TYPE_CHECKING, Any, Dict, Optional, Set, Tuple, Union
 
+import dask.array
 import fsspec
 import numpy as np
 import xarray as xr
@@ -14,12 +14,11 @@ from zarr.errors import GroupNotFoundError, PathNotFoundError
 if TYPE_CHECKING:
     from ..core import EngineHint, FileFormatHint, PathHint, SonarModelsHint
 
-from ..utils.coding import set_time_encodings
-from ..utils.io import check_file_existence, sanitize_file_path
+from ..echodata.utils_platform import _clip_by_time_dim, get_mappings_expanded
+from ..utils.coding import sanitize_dtypes, set_time_encodings
 from ..utils.log import _init_logger
 from ..utils.prov import add_processing_level
 from .convention import sonarnetcdf_1
-from .sensor_ep_version_mapping import ep_version_mapper
 from .widgets.utils import tree_repr
 from .widgets.widgets import _load_static_files, get_template
 
@@ -54,7 +53,6 @@ class EchoData:
         xml_path: Optional["PathHint"] = None,
         sonar_model: Optional["SonarModelsHint"] = None,
         open_kwargs: Optional[Dict[str, Any]] = None,
-        parsed2zarr_obj=None,
     ):
         # TODO: consider if should open datasets in init
         #  or within each function call when echodata is used. Need to benchmark.
@@ -69,32 +67,51 @@ class EchoData:
         self.converted_raw_path: Optional["PathHint"] = converted_raw_path
         self._tree: Optional["DataTree"] = None
 
-        # object associated with directly writing to a zarr file
-        self.parsed2zarr_obj = parsed2zarr_obj
-
         self.__setup_groups()
         # self.__read_converted(converted_raw_path)
 
         self._varattrs = sonarnetcdf_1.yaml_dict["variable_and_varattributes"]
 
+    def cleanup_swap_files(self):
+        """
+        Clean up the swap files during raw data conversion
+        """
+        sonar_group = "Sonar"
+        beam_group_var = "beam_group"
+        for beam_group in self[sonar_group][beam_group_var].to_numpy():
+            # Go through each beam group
+            for var in self[f"{sonar_group}/{beam_group}"].data_vars.values():
+                # Go through each variable and only delete if it's a dask array
+                if isinstance(var.data, dask.array.Array):
+                    da = var.data
+                    # Get the dask graph so we have access to the underlying
+                    # zarr stores
+                    dask_graph = da.__dask_graph__()
+                    # Get the zarr stores
+                    zarr_stores = [
+                        v.store for k, v in dask_graph.items() if "original-from-zarr" in k
+                    ]
+                    fs = zarr_stores[0].fs
+                    from ..utils.io import delete_zarr_store
+
+                    for store in zarr_stores:
+                        delete_zarr_store(store, fs)
+
     def __del__(self):
         # TODO: this destructor seems to not work in Jupyter Lab if restart or
         #  even clear all outputs is used. It will work if you explicitly delete the object
-
-        if (self.parsed2zarr_obj is not None) and (self.parsed2zarr_obj.zarr_file_name is not None):
-            # get Path object of temporary zarr file created by Parsed2Zarr
-            p2z_temp_file = Path(self.parsed2zarr_obj.zarr_file_name)
-
-            # remove temporary directory created by Parsed2Zarr, if it exists
-            if p2z_temp_file.exists():
-                # TODO: do we need to check file permissions here?
-                shutil.rmtree(p2z_temp_file)
+        if self.converted_raw_path is None:
+            # Assumes raw data is in memory
+            self.cleanup_swap_files()
 
     def __str__(self) -> str:
         fpath = "Internal Memory"
         if self.converted_raw_path:
             fpath = self.converted_raw_path
-        return f"<EchoData: standardized raw data from {fpath}>\n{tree_repr(self._tree)}"
+        repr_str = "No data found."
+        if self._tree is not None:
+            repr_str = tree_repr(self._tree)
+        return f"<EchoData: standardized raw data from {fpath}>\n{repr_str}"
 
     def __repr__(self) -> str:
         return str(self)
@@ -158,9 +175,6 @@ class EchoData:
 
         echodata._set_tree(tree)
 
-        # convert to newest echopype version structure, if necessary
-        ep_version_mapper.map_ep_version(echodata)
-
         if isinstance(converted_raw_path, fsspec.FSMap):
             # Convert fsmap to Path so it can be used
             # for retrieving the path strings
@@ -193,16 +207,28 @@ class EchoData:
                 setattr(self, group, node)
 
     @property
-    def version_info(self) -> Tuple[int]:
-        if self["Provenance"].attrs.get("conversion_software_name", None) == "echopype":
-            version_str = self["Provenance"].attrs.get("conversion_software_version", None)
+    def version_info(self) -> Union[Tuple[int], None]:
+        def _get_version_tuple(provenance_type):
+            """
+            Parameters
+            ----------
+            provenance_type : str
+                Either conversion or combination
+            """
+            version_str = self["Provenance"].attrs.get(f"{provenance_type}_software_version", None)
             if version_str is not None:
                 if version_str.startswith("v"):
                     # Removes v in case of v0.4.x or less
                     version_str = version_str.strip("v")
                 version_num = version_str.split(".")[:3]
                 return tuple([int(i) for i in version_num])
-        return None
+
+        if self["Provenance"].attrs.get("combination_software_name", None) == "echopype":
+            return _get_version_tuple("combination")
+        elif self["Provenance"].attrs.get("conversion_software_name", None) == "echopype":
+            return _get_version_tuple("conversion")
+        else:
+            return None
 
     @property
     def nbytes(self) -> float:
@@ -210,12 +236,13 @@ class EchoData:
 
     @property
     def group_paths(self) -> Set[str]:
-        return {i[1:] if i != "/" else "Top-level" for i in self._tree.groups}
+        return tuple(i[1:] if i != "/" else "Top-level" for i in self._tree.groups)
 
     @staticmethod
     def __get_dataset(node: DataTree) -> Optional[xr.Dataset]:
         if node.has_data or node.has_attrs:
-            return node.ds
+            # validate and clean dtypes
+            return sanitize_dtypes(node.ds)
         return None
 
     def __get_node(self, key: Optional[str]) -> DataTree:
@@ -265,43 +292,44 @@ class EchoData:
     def update_platform(
         self,
         extra_platform_data: xr.Dataset,
-        time_dim="time",
+        variable_mappings=Dict[str, str],
         extra_platform_data_file_name=None,
     ):
         """
         Updates the `EchoData["Platform"]` group with additional external platform data.
 
-        `extra_platform_data` must be an xarray Dataset.
-        The name of the time dimension in `extra_platform_data` is specified by the
-        `time_dim` parameter.
-        Data is extracted from `extra_platform_data` by variable name; only the data
-        in `extra_platform_data` with the following variable names will be used:
-            - `"pitch"`
-            - `"roll"`
-            - `"vertical_offset"`
-            - `"latitude"`
-            - `"longitude"`
-            - `"water_level"`
-        The data inserted into the Platform group will be indexed by a dimension named
-        `"time1"`.
+        `extra_platform_data` must be an xarray Dataset. Data is extracted from
+        `extra_platform_data` by variable name. Only data assigned to a pre-existing
+        (but possibly all-nan) variable name in Platform will be processed. These
+        Platform variables include latitude, longitude, pitch, roll, vertical_offset, etc.
+        See the variables present in the EchoData object's Platform group to obtain a
+        complete list of possible variables.
+        Different external variables may be dependent on different time dimensions, but
+        latitude and longitude (if specified) must share the same time dimension. New
+        time dimensions will be added as needed. For example, if variables to be added
+        from the external data use two time dimensions and the Platform group has time
+        dimensions time2 and time2, new dimensions time3 and time4 will be created.
 
         Parameters
         ----------
         extra_platform_data : xr.Dataset
             An `xr.Dataset` containing the additional platform data to be added
             to the `EchoData["Platform"]` group.
-        time_dim: str, default="time"
-            The name of the time dimension in `extra_platform_data`; used for extracting
-            data from `extra_platform_data`.
+        variable_mappings: Dict[str,str]
+            A dictionary mapping Platform variable names (dict key) to the
+            external-data variable name (dict value).
         extra_platform_data_file_name: str, default=None
             File name for source of extra platform data, if read from a file
 
         Examples
         --------
         >>> ed = echopype.open_raw(raw_file, "EK60")
-        >>> extra_platform_data = xr.open_dataset(extra_platform_data_file)
-        >>> ed.update_platform(extra_platform_data,
-        >>>         extra_platform_data_file_name=extra_platform_data_file)
+        >>> extra_platform_data = xr.open_dataset(extra_platform_data_file_name)
+        >>> ed.update_platform(
+        >>>     extra_platform_data,
+        >>>     variable_mappings={"longitude": "lon", "latitude": "lat", "roll": "ROLL"},
+        >>>     extra_platform_data_file_name=extra_platform_data_file_name
+        >>> )
         """
 
         # Handle data stored as a CF Trajectory Discrete Sampling Geometry
@@ -312,114 +340,118 @@ class EchoData:
             and extra_platform_data.attrs["featureType"].lower() == "trajectory"
         ):
             for coordvar in extra_platform_data.coords:
-                if (
-                    "cf_role" in extra_platform_data[coordvar].attrs
-                    and extra_platform_data[coordvar].attrs["cf_role"] == "trajectory_id"
-                ):
+                coordvar_attrs = extra_platform_data[coordvar].attrs
+                if "cf_role" in coordvar_attrs and coordvar_attrs["cf_role"] == "trajectory_id":
                     trajectory_var = coordvar
 
+                if "standard_name" in coordvar_attrs and coordvar_attrs["standard_name"] == "time":
+                    time_dim = coordvar
+
             # assumes there's only one trajectory in the dataset (index 0)
             extra_platform_data = extra_platform_data.sel(
                 {trajectory_var: extra_platform_data[trajectory_var][0]}
             )
             extra_platform_data = extra_platform_data.drop_vars(trajectory_var)
-            extra_platform_data = extra_platform_data.swap_dims({"obs": time_dim})
+            obs_dim = list(extra_platform_data[time_dim].dims)[0]
+            extra_platform_data = extra_platform_data.swap_dims({obs_dim: time_dim})
 
-        # clip incoming time to 1 less than min of EchoData["Sonar/Beam_group1"]["ping_time"] and
-        #   1 greater than max of EchoData["Sonar/Beam_group1"]["ping_time"]
-        # account for unsorted external time by checking whether each time value is between
-        #   min and max ping_time instead of finding the 2 external times corresponding to the
-        #   min and max ping_time and taking all the times between those indices
-        sorted_external_time = extra_platform_data[time_dim].data
-        sorted_external_time.sort()
-        # fmt: off
-        min_index = max(
-            np.searchsorted(
-                sorted_external_time, self["Sonar/Beam_group1"]["ping_time"].min(), side="left"
-            ) - 1,
-            0,
-        )
-        # fmt: on
-        max_index = min(
-            np.searchsorted(
-                sorted_external_time,
-                self["Sonar/Beam_group1"]["ping_time"].max(),
-                side="right",
-            ),
-            len(sorted_external_time) - 1,
-        )
-        extra_platform_data = extra_platform_data.sel(
-            {
-                time_dim: np.logical_and(
-                    sorted_external_time[min_index] <= extra_platform_data[time_dim],
-                    extra_platform_data[time_dim] <= sorted_external_time[max_index],
-                )
-            }
-        )
-
-        platform = self["Platform"]
-        platform = platform.drop_dims(["time1"], errors="ignore")
-        # drop_dims is also dropping latitude, longitude and sentence_type why?
-        platform = platform.assign_coords(time1=extra_platform_data[time_dim].values)
+        # History attribute to be included in each updated variable
         history_attr = f"{datetime.datetime.utcnow()} +00:00. Added from external platform data"
         if extra_platform_data_file_name:
             history_attr += ", from file " + extra_platform_data_file_name
-        time1_attrs = {
-            **self._varattrs["platform_coord_default"]["time1"],
-            **{"history": history_attr},
-        }
-        platform["time1"] = platform["time1"].assign_attrs(**time1_attrs)
 
-        platform_vars_sourcenames = {
-            "pitch": ["pitch", "PITCH"],
-            "roll": ["roll", "ROLL"],
-            "vertical_offset": ["heave", "HEAVE", "vertical_offset", "VERTICAL_OFFSET"],
-            "latitude": ["lat", "latitude", "LATITUDE"],
-            "longitude": ["lon", "longitude", "LONGITUDE"],
-            "water_level": ["water_level", "WATER_LEVEL"],
-        }
+        platform = self["Platform"]
 
-        dropped_vars_target = platform_vars_sourcenames.keys()
-        dropped_vars = []
-        for var in dropped_vars_target:
-            if var in platform and (~platform[var].isnull()).all():
-                dropped_vars.append(var)
-        if len(dropped_vars) > 0:
-            logger.warning(
-                f"Some variables in the original Platform group will be overwritten: {', '.join(dropped_vars)}"  # noqa
-            )
-        platform = platform.drop_vars(
-            dropped_vars_target,
-            errors="ignore",
+        # Retain only variable_mappings items where
+        # either the Platform group or extra_platform_data
+        # contain the corresponding variables or contain valid (not all nan) data
+        mappings_expanded = get_mappings_expanded(
+            logger, extra_platform_data, variable_mappings, platform
         )
 
-        def mapping_search_variable(mapping, keys, default=None):
-            for key in keys:
-                if key in mapping:
-                    return mapping[key].data
-            return default
+        # Create names for required new time dimensions
+        ext_time_dims = list(
+            {
+                v["ext_time_dim_name"]
+                for v in mappings_expanded.values()
+                if v["ext_time_dim_name"] != "scalar"
+            }
+        )
+        time_dims_max = max([int(dim[-1]) for dim in platform.dims if dim.startswith("time")])
+        new_time_dims = [f"time{time_dims_max + i + 1}" for i in range(len(ext_time_dims))]
+        # Map each new time dim name to the external time dim name:
+        new_time_dims_mappings = {new: ext for new, ext in zip(new_time_dims, ext_time_dims)}
 
-        num_obs = len(extra_platform_data[time_dim])
-        for platform_var, sourcenames in platform_vars_sourcenames.items():
-            platform = platform.update(
-                {
-                    platform_var: (
-                        "time1",
-                        mapping_search_variable(
-                            extra_platform_data,
-                            sourcenames,
-                            platform.get(platform_var, np.full(num_obs, np.nan)),
-                        ),
-                    )
-                }
+        # Process variable updates by corresponding new time dimensions
+        for time_dim in new_time_dims:
+            ext_time_dim = new_time_dims_mappings[time_dim]
+            mappings_selected = {
+                k: v for k, v in mappings_expanded.items() if v["ext_time_dim_name"] == ext_time_dim
+            }
+            ext_vars = [v["external_var"] for v in mappings_selected.values()]
+            ext_ds = _clip_by_time_dim(
+                extra_platform_data[ext_vars], ext_time_dim, self["Sonar/Beam_group1"]["ping_time"]
             )
 
-        for var in dropped_vars_target:
-            var_attrs = self._varattrs["platform_var_default"][var]
-            # Insert history attr only if the variable was inserted with valid values
-            if not platform[var].isnull().all():
-                var_attrs["history"] = history_attr
-            platform[var] = platform[var].assign_attrs(**var_attrs)
+            # Create new time coordinate and dimension
+            platform = platform.assign_coords(**{time_dim: ext_ds[ext_time_dim].values})
+            time_attrs = {
+                "axis": "T",
+                "standard_name": "time",
+                "long_name": "Timestamps from an external dataset",
+                "comment": "Time coordinate originated from a dataset "
+                "external to the sonar data files.",
+                "history": f"{history_attr}. From external {ext_time_dim} variable.",
+            }
+            platform[time_dim] = platform[time_dim].assign_attrs(**time_attrs)
+
+            # Process each platform variable that will be replaced
+            for platform_var in mappings_selected.keys():
+                ext_var = mappings_expanded[platform_var]["external_var"]
+                platform_var_attrs = platform[platform_var].attrs.copy()
+
+                # Create new (replaced) variable using dataset "update"
+                # With update, dropping the variable first is not needed
+                platform = platform.update({platform_var: (time_dim, ext_ds[ext_var].data)})
+
+                # Assign attributes to newly created (replaced) variables
+                var_attrs = platform_var_attrs
+                var_attrs["history"] = f"{history_attr}. From external {ext_var} variable."
+                platform[platform_var] = platform[platform_var].assign_attrs(**var_attrs)
+
+        # Update scalar variables, if any
+        scalar_vars = [
+            platform_var
+            for platform_var, v in mappings_expanded.items()
+            if v["ext_time_dim_name"] == "scalar"
+        ]
+        for platform_var in scalar_vars:
+            # Set timestamp equal to the first ping time whenever either
+            # latitude or longitude is updated without a time dimension
+            ext_var = mappings_expanded[platform_var]["external_var"]
+            if platform_var == "latitude" or platform_var == "longitude":
+                platform[platform_var].data = np.array([extra_platform_data[ext_var].data])
+                platform[platform_var] = platform[platform_var].assign_coords(
+                    **{
+                        platform[platform_var].dims[0]: [
+                            self["Sonar/Beam_group1"]["ping_time"].data[0]
+                        ]
+                    }
+                )
+            else:
+                # Replace the scalar value and add a history attribute
+                platform[platform_var].data = float(extra_platform_data[ext_var].data)
+            platform[platform_var] = platform[platform_var].assign_attrs(
+                **{"history": f"{history_attr}. From external {ext_var} variable."}
+            )
+
+        # Drop pre-existing time dimensions that are no longer being used
+        used_dims = {
+            platform[platform_var].dims[0]
+            for platform_var in platform.data_vars
+            if len(platform[platform_var].dims) > 0
+        }
+        platform = platform.drop_dims(set(platform.dims).difference(used_dims), errors="ignore")
 
         self["Platform"] = set_time_encodings(platform)
 
@@ -455,11 +487,15 @@ class EchoData:
 
     def _check_path(self, filepath: "PathHint"):
         """Check if converted_raw_path exists"""
+        from ..utils.io import check_file_existence
+
         file_exists = check_file_existence(filepath, self.storage_options)
         if not file_exists:
             raise FileNotFoundError(f"There is no file named {filepath}")
 
     def _sanitize_path(self, filepath: "PathHint") -> "PathHint":
+        from ..utils.io import sanitize_file_path
+
         filepath = sanitize_file_path(filepath, self.storage_options)
         return filepath
 
